@@ -8,12 +8,12 @@
 #
 ###########################################################################
 
-import string
 import ldap
 import ldif,dsml
 import copy
 import base64
 import StringIO
+from sets import Set
 
 from base.backend.ObjectClassAttributeInfo import ObjectClassAttributeInfo
 from base.utils import stripSpecialChars
@@ -31,6 +31,7 @@ class SmartDataObject (object):
     def __init__(self, data, serverMeta):
         self.doSchemaChecks = True
         self.isValid = False
+        self.checkErrorMessageList = ["No error checking done yet."]
         
         self.dn = data[0]
         self.data = data[1]
@@ -39,7 +40,7 @@ class SmartDataObject (object):
         # Important for lower- and uppercase variants
         self.objectClassName = None
         for x in self.data.keys():
-                if "objectclass" == string.lower(x):
+                if "objectclass" == x.lower():
                     self.objectClassName = x
                     break
         
@@ -116,7 +117,7 @@ class SmartDataObject (object):
             #return None
             return []
         else:
-            return self.data[self.objectClassName]
+            return self.data[self.objectClassName][:]
             
 ###############################################################################
 
@@ -127,7 +128,7 @@ class SmartDataObject (object):
         structList = []
         
         for x in self.getObjectClasses():
-            if self.serverSchema.isStructural(x):
+            if self.isObjectclassStructural(x):
                 structList.append(x)
                 
         return structList
@@ -139,7 +140,7 @@ class SmartDataObject (object):
         """
         
         for x in self.getObjectClasses():
-            if self.serverSchema.isStructural(x):
+            if self.isObjectclassStructural(x):
                 return True
                 
         return False
@@ -222,6 +223,11 @@ class SmartDataObject (object):
         else:
             return None
             
+###############################################################################
+
+    def getAttributeListForObjectClass(self, objectClass):
+        return self.serverSchema.getAttributeListForObjectClass(objectClass)
+    
 ###############################################################################
 
     def addAttributeValue(self, attributeName=None, valueList=None, replaceOldValues=False):
@@ -372,7 +378,7 @@ class SmartDataObject (object):
                 errorList.append(" Current data: " + repr(self.data))
                         
                 raise LdapDataException("".join(errorList))
-        elif "rdn" == string.lower(attributeName):
+        elif "rdn" == attributeName.lower():
             self.setDN(newValue)
         else:
             errorList = []
@@ -394,10 +400,10 @@ class SmartDataObject (object):
             
         mustSet, maySet = self.getPossibleAttributes()
         attributeList = mustSet.union(maySet)
-        attributeName = string.lower(attributeName)
+        attributeName = attributeName.lower()
         
         for x in attributeList:
-            if attributeName == string.lower(x):
+            if attributeName == x.lower():
                 return True
                 
         return False
@@ -450,7 +456,7 @@ class SmartDataObject (object):
         if None == attributeName:
             raise FunctionArgumentException("Function isAttributeImage(attributeName) called without correct parameters.")
             
-        if "jpegphoto" == string.lower(attributeName):
+        if "jpegphoto" == attributeName.lower():
             return True
         else:
             return False
@@ -464,7 +470,7 @@ class SmartDataObject (object):
         if None == attributeName:
             raise FunctionArgumentException("Function isAttributePassword(attributeName) called without correct parameters.")
           
-        tmpName = string.lower(attributeName)
+        tmpName = attributeName.lower()
         if tmpName in self.passwordList:
             return True
         else:
@@ -479,7 +485,7 @@ class SmartDataObject (object):
         """
         
         for x in self.getObjectClasses():
-            if "alias" == string.lower(x):
+            if "alias" == x.lower():
                 return True
             
         return False
@@ -569,7 +575,27 @@ class SmartDataObject (object):
 ###############################################################################
 
     def checkIntegrity(self):
-        pass
+        isOk = True
+        errorList = []
+        
+        # First check for objectClasses
+        objectClasses = self.getObjectClasses()
+        
+        for x in objectClasses:
+            if not self.serverSchema.hasObjectClass(x):
+                isOk = False
+                errorList.append("ObjectClass " + x + " not present in server schema.")
+                
+        # Now check if all attributes are present in schema
+        attributes = self.getAttributeList()
+        
+        for x in attributes:
+            if not self.serverSchema.hasAttribute(x):
+                isOk = False
+                errorList.append("Attribute " + x + " not present in server schema.")
+        
+        self.isValid = isOk
+        self.checkErrorMessageList = errorList
         
 ###############################################################################
 
@@ -657,13 +683,13 @@ class SmartDataObject (object):
         """ Add an objectClass to the object.
         """
         
-        className = string.lower(className)
+        className = className.lower()
         className = className.encode('utf-8')
         
         if self.hasObjectClass(className):
             return
             
-        if self.serverSchema.isStructural(className):
+        if self.isObjectclassStructural(className):
             structList = self.getStructuralClasses()
             for x in structList:
                 if not self.serverSchema.sameObjectClassChain(className, x):
@@ -673,8 +699,8 @@ class SmartDataObject (object):
                     errorList.append(" Current objectClasses: " + repr(self.getObjectClasses()))
                         
                     raise LdapDataException("".join(errorList))
-                
-        self.data['objectClass'].append(className)
+        
+        self.data[self.objectClassName].append(className)
             
 ###############################################################################
 
@@ -682,10 +708,10 @@ class SmartDataObject (object):
         """ Checks if the current object has the objectClass className.
         """
         
-        className = string.lower(className)
+        className = className.lower()
         
         for x in self.getObjectClasses():
-            if className == string.lower(x):
+            if className == x.lower():
                 return True
                 
         return False
@@ -696,10 +722,10 @@ class SmartDataObject (object):
         """ Check if the current object has the attribute attributeName.
         """
         
-        attributeName = string.lower(attributeName)
+        attributeName = attributeName.lower()
         
         for x in self.getAttributeList():
-            if attributeName == string.lower(x):
+            if attributeName == x.lower():
                 return True
                 
         return False
@@ -707,7 +733,71 @@ class SmartDataObject (object):
 ###############################################################################
 
     def deleteObjectClass(self, className):
-        pass
+        className = className.lower()
+
+        found = False
+        for x in self.getObjectClasses():
+            if className == x.lower():
+                className = x
+                found = True
+                break
+        if not found:
+            return
+            
+        
+        # We have to make sure that one structural class remains after deleting
+        if self.isObjectclassStructural(className):
+            classList = self.getObjectClasses()
+            classList.remove(className)
+            tmpList = self.getObjectClassChain(className, classList)
+            if len(tmpList) == 0:
+                errorList = []
+                errorList.append("Can't delete objectClass to object. ObjectClass " + className + " is the last structural Class and can't be deleted.")
+                errorList.append(" DN: " + self.getPrettyDN() + ".")
+                errorList.append(" Current objectClasses: " + repr(self.getObjectClasses()))
+                        
+                raise LdapDataException("".join(errorList))
+
+        self.data[self.objectClassName].remove(className)
+
+        must, may = self.getPossibleAttributes()
+        all = Set(self.getAttributeList())
+        rest = all - must.union(may)
+        for x in rest:
+            self.deleteAttribute(x)
+        
+###############################################################################
+
+    def getObsoleteAttributes(self, className):
+        """ Returns list of attributes which will be removed when className 
+            is removed from the list of objectClasses.
+        """
+        
+        attributeList = self.serverSchema.getAttributeListForObjectClass(className)
+        currentList = self.getAttributeList()
+        
+        tmpList = []
+        for x in currentList:
+            if x in attributeList:
+                tmpList.append(x)
+                
+                
+        classList = self.getObjectClasses()
+        classList.remove(className)
+        
+        attributeList = tmpList[:]
+        for x in tmpList:
+            removeAttribute = False
+            for y in classList:
+                # Attribute is provided by another objectclass and there is no 
+                # need to remove it.
+                if x in self.serverSchema.getAttributeListForObjectClass(y):
+                    removeAttribute = True
+            
+            if removeAttribute:
+                attributeList.remove(x)
+        
+        return attributeList
         
 ###############################################################################
 
@@ -782,6 +872,21 @@ class SmartDataObject (object):
         
         return self.serverSchema.getAllAttributes(self.getObjectClasses())
         
+###############################################################################
+
+    def isObjectclassStructural(self, objclass):
+        """ Returns True if the objectclass is structural"""
+    
+        return self.serverSchema.isStructural(objclass)
+
+###############################################################################
+
+    def getObjectClassChain(self, className, classList):
+        """ Returns a list of objectClasses which belong to the same chain 
+            as className, given by classList.
+        """
+
+        return self.serverSchema.getObjectClassChain(className, classList)
         
 ###############################################################################
 ###############################################################################
