@@ -22,6 +22,7 @@ import environment
 from base.gui.BaseSelector import BaseSelector
 from base.backend.LumaConnection import LumaConnection
 from base.utils.gui.LumaErrorDialog import LumaErrorDialog
+from base.utils.backend.LogObject import LogObject
 
 class ServerDialog(ServerDialogDesign):
     """The dialog for managing all server information.
@@ -36,6 +37,9 @@ class ServerDialog(ServerDialogDesign):
         self._PREFIX = environment.lumaInstallationPrefix
         
         self.iconPath = os.path.join(self._PREFIX, "share", "luma", "icons")
+        folderPixmap = QPixmap(os.path.join(self.iconPath, "folder.png"))
+        self.certFileButton.setPixmap(folderPixmap)
+        self.certKeyFileButton.setPixmap(folderPixmap)
         self.networkLabel.setPixmap(QPixmap(os.path.join(self.iconPath, "worldmedium.png")))
         self.authLabel.setPixmap(QPixmap(os.path.join(self.iconPath, "passwordmedium.png")))
         self.guiParent = parent
@@ -68,6 +72,8 @@ class ServerDialog(ServerDialogDesign):
         
         self.disableBaseLookup = False
         
+        self.originalBackGroundColor = self.certFileEdit.paletteBackgroundColor()
+        
         self.displayServerList()
         
 ###############################################################################
@@ -91,10 +97,7 @@ class ServerDialog(ServerDialogDesign):
         """ Change server information if another server has been selected.
         """
         
-        #selectedServerString = unicode(self.serverIconView.currentItem().text())
         selectedServerString = unicode(tmpItem.text(0))
-        
-        
         x = self.serverListObject.getServerObject(selectedServerString)
         self.currentServer = x
         
@@ -103,17 +106,47 @@ class ServerDialog(ServerDialogDesign):
         self.bindAnonBox.blockSignals(True)
         self.bindLineEdit.blockSignals(True)
         self.passwordLineEdit.blockSignals(True)
-        self.tlsCheckBox.blockSignals(True)
         self.methodBox.blockSignals(True)
         self.aliasBox.blockSignals(True)
+        self.validateBox.blockSignals(True)
+        self.useClientCertBox.blockSignals(True)
+        self.certFileEdit.blockSignals(True)
+        self.certKeyfileEdit.blockSignals(True)
         
         self.hostLineEdit.setText(x.host)
         self.portSpinBox.setValue(x.port)
         self.bindAnonBox.setChecked(int(x.bindAnon))
         self.bindLineEdit.setText(x.bindDN)
         self.passwordLineEdit.setText(x.bindPassword)
-        self.tlsCheckBox.setChecked(int(x.tls))
-        #self.methodBox.setCurrentItem(self.authentificationMethods.index(x.authMethod))
+        
+        if x.encryptionMethod == u"None":
+            self.encryptionBox.setCurrentItem(0)
+            self.validateBox.setEnabled(False)
+            self.useClientCertBox.setEnabled(False)
+        elif x.encryptionMethod == u"TLS":
+            self.encryptionBox.setCurrentItem(1)
+            self.validateBox.setEnabled(True)
+            self.useClientCertBox.setEnabled(True)
+        elif x.encryptionMethod == u"SSL":
+            self.encryptionBox.setCurrentItem(2)
+            self.validateBox.setEnabled(True)
+            self.useClientCertBox.setEnabled(True)
+            
+        if x.checkServerCertificate == u"never":
+            self.validateBox.setCurrentItem(0)
+        elif x.checkServerCertificate == u"allow":
+            self.validateBox.setCurrentItem(1)
+        elif x.checkServerCertificate == u"try":
+            self.validateBox.setCurrentItem(2)
+        elif x.checkServerCertificate == u"demand":
+            self.validateBox.setCurrentItem(3)
+            
+        self.useClientCertBox.setChecked(x.useCertificate)
+        self.certFileEdit.setText(x.clientCertFile)
+        self.certKeyfileEdit.setText(x.clientCertKeyfile)
+        
+        self.enableClientCertWidgets(x.useCertificate)
+        
         self.methodBox.setCurrentText(x.authMethod)
         self.bindAnonChanged(x.bindAnon, True)
         self.aliasBox.setChecked(int(x.followAliases))
@@ -123,9 +156,12 @@ class ServerDialog(ServerDialogDesign):
         self.bindAnonBox.blockSignals(False)
         self.bindLineEdit.blockSignals(False)
         self.passwordLineEdit.blockSignals(False)
-        self.tlsCheckBox.blockSignals(False)
         self.methodBox.blockSignals(False)
         self.aliasBox.blockSignals(False)
+        self.validateBox.blockSignals(False)
+        self.useClientCertBox.blockSignals(False)
+        self.certFileEdit.blockSignals(False)
+        self.certKeyfileEdit.blockSignals(False)
         
         authMethod = self.currentServer.authMethod
         if (authMethod == u"SASL GSSAPI") or (authMethod == u"SASL EXTERNAL") or x.bindAnon :
@@ -135,7 +171,7 @@ class ServerDialog(ServerDialogDesign):
             self.passwordLineEdit.setEnabled(True)
             self.bindLineEdit.setEnabled(True)
             
-        self.manageBaseBaseButton.setEnabled(not self.currentServer.autoBase)
+        self.manageBaseButton.setEnabled(not self.currentServer.autoBase)
         self.baseBox.blockSignals(True)
         self.baseBox.setChecked(self.currentServer.autoBase)
         self.baseBox.blockSignals(False)
@@ -253,16 +289,34 @@ class ServerDialog(ServerDialogDesign):
 ###############################################################################
 
     def tlsChanged(self, tmpBool):
-        self.applyButton.setEnabled(1)
-        
         tlsBool = self.tlsCheckBox.isChecked()
         
         if tlsBool:
+            # Set port value
             self.portSpinBox.setValue(636)
+            
+            # Enable certificate widgets
+            self.useClientCertBox.setEnabled(True)
+            
+            tmpBool = self.useClientCertBox.isChecked()
+            
+            self.certFileEdit.setEnabled(tmpBool)
+            self.certKeyfileEdit.setEnabled(tmpBool)
+            self.certFileButton.setEnabled(tmpBool)
+            self.certKeyFileButton.setEnabled(tmpBool)
         else:
+            # Set port value
             self.portSpinBox.setValue(389)
             
+            # Enable certificate widgets
+            self.useClientCertBox.setEnabled(False)
+            self.certFileEdit.setEnabled(False)
+            self.certKeyfileEdit.setEnabled(False)
+            self.certFileButton.setEnabled(False)
+            self.certKeyFileButton.setEnabled(False)
+            
         self.currentServer.tls = tlsBool
+        self.applyButton.setEnabled(True)
             
 ###############################################################################
 
@@ -327,7 +381,6 @@ class ServerDialog(ServerDialogDesign):
         self.passwordLineEdit.blockSignals(True)
         
         authMethod = self.currentServer.authMethod
-        print authMethod
         if (authMethod == u"SASL GSSAPI") or (u"SASL EXTERNAL" == authMethod):
             self.passwordLineEdit.setEnabled(False)
             self.bindLineEdit.setEnabled(False)
@@ -350,7 +403,7 @@ class ServerDialog(ServerDialogDesign):
         self.applyButton.setEnabled(True)
         automaticBase = self.baseBox.isChecked()
         self.currentServer.autoBase = automaticBase
-        self.manageBaseBaseButton.setEnabled(not automaticBase)
+        self.manageBaseButton.setEnabled(not automaticBase)
         self.displayBase()
         
 ###############################################################################
@@ -392,3 +445,167 @@ class ServerDialog(ServerDialogDesign):
     def aliasesChanged(self):
         self.applyButton.setEnabled(True)
         self.currentServer.followAliases = self.aliasBox.isChecked()
+
+###############################################################################
+
+    def certFileChanged(self, tmpFileName):
+        tmpFileName = unicode(tmpFileName)
+        
+        fileWarning = False
+        # Now do file checking
+        if os.path.isdir(tmpFileName):
+            fileWarning = True
+        else:
+            try:
+                if os.path.isfile(tmpFileName) or os.path.islink(tmpFileName):
+                    open(tmpFileName, "r")
+                else:
+                    fileWarning = True
+            except IOError, e:
+                fileWarning = True
+                
+        if tmpFileName == "":
+            fileWarning = False
+        
+        if fileWarning:
+            self.certFileEdit.setPaletteBackgroundColor(Qt.red)
+        else:
+            self.certFileEdit.unsetPalette()
+        
+        # Now do internal stuff like updating the ServerObject 
+        # and activate apply button
+        self.currentServer.clientCertFile = tmpFileName
+        self.applyButton.setEnabled(True)
+        
+###############################################################################
+
+    def certKeyFileChanged(self, tmpFileName):
+        tmpFileName = unicode(tmpFileName)
+        
+        fileWarning = False
+        # Now do file checking
+        if os.path.isdir(tmpFileName):
+            fileWarning = True
+        else:
+            try:
+                if os.path.isfile(tmpFileName) or os.path.islink(tmpFileName):
+                    open(tmpFileName, "r")
+                else:
+                    fileWarning = True
+            except IOError, e:
+                fileWarning = True
+                
+        if tmpFileName == "":
+            fileWarning = False
+        
+        if fileWarning:
+            self.certKeyfileEdit.setPaletteBackgroundColor(Qt.red)
+        else:
+            self.certKeyfileEdit.unsetPalette()
+        
+        # Now do internal stuff like updating the ServerObject 
+        # and activate apply button
+        self.currentServer.clientCertKeyfile = tmpFileName
+        self.applyButton.setEnabled(True)
+        
+###############################################################################
+
+    def showCertFileDialog(self):
+        filename = QFileDialog.getOpenFileName(\
+            None,
+            None,
+            None, None,
+            self.trUtf8("Select certificate file"),
+            None, 1)
+            
+        self.certFileEdit.setText(unicode(filename))
+
+        
+###############################################################################
+
+    def showCertKeyFileDialog(self):
+        filename = QFileDialog.getOpenFileName(\
+            None,
+            None,
+            None, None,
+            self.trUtf8("Select certificate key file"),
+            None, 1)
+            
+        self.certKeyfileEdit.setText(unicode(filename))
+        
+###############################################################################
+
+    def encryptionChanged(self, typeNumber):
+        encryptionMethod = u"None"
+        
+        if typeNumber == 0:
+            encryptionMethod = u"None"
+        elif typeNumber == 1:
+            encryptionMethod = u"TLS"
+        elif typeNumber == 2:
+            encryptionMethod = u"SSL"
+        
+        self.currentServer.encryptionMethod = encryptionMethod
+        
+        tmpBool = False
+        if typeNumber > 0:
+            tmpBool = True
+        self.validateBox.setEnabled(tmpBool)
+        self.useClientCertBox.setEnabled(tmpBool)
+        
+        if self.currentServer.useCertificate:
+            self.enableClientCertWidgets(True)
+        else:
+            self.enableClientCertWidgets(False)
+        
+        # Set port numbers according to the encryption method
+        self.portSpinBox.blockSignals(True)
+        
+        portValue = 389
+        if encryptionMethod == u"SSL":
+            portValue = 636
+            
+        self.portSpinBox.setValue(portValue)
+        self.currentServer.port = portValue
+            
+        self.portSpinBox.blockSignals(True)
+        
+        self.applyButton.setEnabled(True)
+        
+###############################################################################
+
+    def validityCheckChanged(self, typeNumber):
+        validityType = u"demand"
+        
+        if typeNumber == 0:
+            validityType = u"never"
+        elif typeNumber == 1:
+            validityType = u"allow"
+        elif typeNumber == 2:
+            validityType = u"try"
+        elif typeNumber == 3:
+            validityType = u"demand"
+            
+        self.currentServer.checkServerCertificate = validityType
+        self.applyButton.setEnabled(True)
+        
+###############################################################################
+
+    def enableClientCerts(self, toggleBool):
+        if toggleBool >= 1:
+            self.currentServer.useCertificate = True
+        else:
+            self.currentServer.useCertificate = False
+            
+        self.applyButton.setEnabled(True)
+        self.enableClientCertWidgets(toggleBool)
+        
+###############################################################################
+
+    def enableClientCertWidgets(self, enableBool):
+        self.certFileEdit.setEnabled(enableBool)
+        self.certKeyfileEdit.setEnabled(enableBool)
+        self.certFileButton.setEnabled(enableBool)
+        self.certKeyFileButton.setEnabled(enableBool)
+    
+    
