@@ -150,35 +150,58 @@ class AdvancedObjectWidget(QWidget):
             self.enableToolButtons(False)
             return 
         
-        tmpList = []
-        tmpList.append("<html>")
-        tmpList.append("""<body>""")
-        tmpList.append("""<table border="0" cellpadding="1" cellspacing="0" width="100%">""")
-        tmpList.append("""<tr>""")
-        tmpList.append("""<td bgcolor="#B2CAE7" width="40%"><font size="+1"> <b>Distinguished Name:</b> </font></td>""")
-        tmpList.append("""<td bgcolor="#B2CAE7" width="60%"><font size="+1"><b>""" + self.ldapDataObject.getPrettyDN() + """</b></font></td>""")
+        self.ldapDataObject.checkIntegrity()
         
-        if self.CREATE:
-            self.mimeFactory.setPixmap("editPixmap", self.editPixmap)
-            tmpList.append("""<td width=5%><a name=RDN__0__edit><img source="editPixmap"></a></td>""")
+        if self.ldapDataObject.isValid:
+            tmpList = []
+            tmpList.append("<html>")
+            tmpList.append("""<body>""")
+            tmpList.append("""<table border="0" cellpadding="1" cellspacing="0" width="100%">""")
+            tmpList.append("""<tr>""")
+            tmpList.append("""<td bgcolor="#B2CAE7" width="40%"><font size="+1"> <b>Distinguished Name:</b> </font></td>""")
+            tmpList.append("""<td bgcolor="#B2CAE7" width="60%"><font size="+1"><b>""" + self.ldapDataObject.getPrettyDN() + """</b></font></td>""")
         
-        tmpList.append("""</tr>""")
+            if self.CREATE:
+                self.mimeFactory.setPixmap("editPixmap", self.editPixmap)
+                tmpList.append("""<td width=5%><a name=RDN__0__edit><img source="editPixmap"></a></td>""")
         
-        tmpList.append("</table>")
-        tmpList.append("<br>")
+            tmpList.append("""</tr>""")
         
-        tmpList.append(self.createClassString())
+            tmpList.append("</table>")
+            tmpList.append("<br>")
         
-        tmpList.append("<br>")
+            tmpList.append(self.createClassString())
         
-        tmpList.append(self.createAttributeString())
+            tmpList.append("<br>")
         
-        tmpList.append("</body>")
-        tmpList.append("</html>")
+            tmpList.append(self.createAttributeString())
         
-        self.currentDocument = ("".join(tmpList))
+            tmpList.append("</body>")
+            tmpList.append("</html>")
+        
+            self.currentDocument = ("".join(tmpList))
+        
+            self.objectWidget.setText(self.currentDocument)
+        else:
+            tmpList = []
+            tmpList.append("<html>")
+            tmpList.append("""<body>""")
+            tmpList.append("""<table border="0" cellpadding="1" cellspacing="0" width="100%">""")
+            tmpList.append("""<tr>""")
+            tmpList.append("""<td <font size="+1"> """ + unicode(self.trUtf8("<b>Could not display ldap entry. Reason:</b>")) + """</font></td>""")
+            tmpList.append("""</tr>""")
+            tmpList.append("""<tr>""")
+            tmpList.append("""</tr>""")
+            for x in self.ldapDataObject.checkErrorMessageList:
+                tmpList.append("""<tr>""")
+                tmpList.append("""<td>""" + x + """</td>""")
+                tmpList.append("""</tr>""")
+                
+            tmpList.append("</body>")
+            tmpList.append("</html>")
+            
+            self.objectWidget.setText("".join(tmpList))
         
-        self.objectWidget.setText(self.currentDocument)
         
         self.enableToolButtons(True)
         environment.setBusy(False)
@@ -193,10 +216,42 @@ class AdvancedObjectWidget(QWidget):
         tmpList.append("""<td bgcolor="#C4DFFF" align="center"><b>ObjectClasses</b></td>""")
         tmpList.append("""</tr>""")
         
+        rdn = self.ldapDataObject.getPrettyRDN()
+        rdnClass = rdn.split("=")[0]
+        
         for x in self.ldapDataObject.getObjectClasses():
+            classString = x[:]
+            if self.ldapDataObject.isObjectclassStructural(x):
+                classString = "<b>" + classString + "</b>"
             tmpList.append("""<tr>""")
-            tmpList.append("""<td colspan=2 bgcolor="#E5E5E5" width="100%">""" + x + """</td>""")
-            tmpList.append("""</tr>""")
+            tmpList.append("""<td colspan=2 bgcolor="#E5E5E5" width="100%">""")
+            tmpList.append(classString)
+            
+            allowDelete = True
+            if self.ldapDataObject.isObjectclassStructural(x):
+                classList = self.ldapDataObject.getObjectClasses()
+                classList.remove(x)
+                if len(self.ldapDataObject.getObjectClassChain(x, classList)) == 0:
+                    allowDelete = False
+                    
+            if rdnClass in self.ldapDataObject.getAttributeListForObjectClass(x):
+                allowDelete = False
+                
+                # Now we check if another objectclass provides the rdn attribute
+                classList = self.ldapDataObject.getObjectClasses()
+                classList.remove(x)
+                for y in classList:
+                    if rdnClass in self.ldapDataObject.getAttributeListForObjectClass(y):
+                        allowDelete = True
+                        break
+            
+            if allowDelete and (not (x == 'top')):
+                deleteName = x + "__delete\""
+                self.mimeFactory.setPixmap("deletePixmap", self.deletePixmap)
+                tmpList.append(""" <a name=\"""" + deleteName + """><img source="deletePixmap"></a>""")
+            
+            
+            tmpList.append("""</td></tr>""")
         
         tmpList.append("""</table>""")
         
@@ -343,7 +398,7 @@ class AdvancedObjectWidget(QWidget):
     def getAttributeModifierString(self, univAttributeName, allowDelete, attributeBinaryExport, attributeModify):
         tmpList = []
         
-        tmpList.append("""<td width=10%>""")
+        tmpList.append("""<td width=20%>""")
         
         if attributeModify:
         #if True:
@@ -373,19 +428,22 @@ class AdvancedObjectWidget(QWidget):
         nameString = unicode(nameString)
         tmpList = nameString.split("__")
         
-        if not len(tmpList) == 3:
-            return
-            
-        attributeName = tmpList[0]
-        index = int(tmpList[1])
-        operation = tmpList[2]
+        if tmpList[0] in self.ldapDataObject.getObjectClasses():
+            self.deleteObjectClass(tmpList[0])
+        else:
+            if not len(tmpList) == 3:
+                return
+
+            attributeName = tmpList[0]
+            index = int(tmpList[1])
+            operation = tmpList[2]
         
-        if operation == "edit":
-            self.editAttribute(attributeName, index)
-        elif operation == "delete":
-            self.deleteAttribute(attributeName, index)
-        elif operation == "export":
-            self.exportAttribute(attributeName, index)
+            if operation == "edit":
+                self.editAttribute(attributeName, index)
+            elif operation == "delete":
+                self.deleteAttribute(attributeName, index)
+            elif operation == "export":
+                self.exportAttribute(attributeName, index)
 
 ###############################################################################
 
@@ -449,6 +507,13 @@ class AdvancedObjectWidget(QWidget):
         self.EDITED = True
         self.displayValues()
         
+###############################################################################
+    
+    def deleteObjectClass(self, className):
+        self.ldapDataObject.deleteObjectClass(className)
+        self.EDITED = True
+        self.displayValues()
+        
         
 ###############################################################################
         
@@ -681,8 +746,9 @@ class AdvancedObjectWidget(QWidget):
         lumaConnection.unbind()
         
         if success:
-            itemString = self.ldapDataObject.getPrettyParentDN() + "," + self.ldapDataObject.getServerAlias()
-            self.emit(PYSIGNAL("REOPEN_PARENT"), (itemString,))
+            serverName = self.ldapDataObject.getServerAlias()
+            dn = self.ldapDataObject.getPrettyParentDN()
+            self.emit(PYSIGNAL("REOPEN_PARENT"), (serverName, dn,))
             self.clearView()
             self.enableToolButtons(False)
         else:
