@@ -1,16 +1,52 @@
 # -*- coding: utf-8 -*-
 
 ###########################################################################
-#    Copyright (C) 2003 by Wido Depping
+#    Copyright (C) 2005 by Wido Depping
 #    <widod@users.sourceforge.net>
 #
 # Copyright: See COPYING file that comes with this distribution
 #
 ###########################################################################
 
-from base.backend.ServerObject import ServerObject
-import environment
+from qt import *
+import os.path
 from base.utils.backend.LogObject import LogObject
+import environment
+
+lumaPlugin = True
+pluginName = "usermanagement"
+pluginUserString = qApp.trUtf8("Usermanagement")
+version = ""
+author = "Wido Depping <wido.depping@gmail.com>"
+
+def getIcon(iconPath):
+    try:
+        iconPixmap = QPixmap (os.path.join (iconPath, "plugin.png"))
+    except:
+        print "Debug: Icon for plugin " + pluginName + " could not be opened."
+        return None
+
+    return iconPixmap
+    
+###############################################################################
+
+def getPluginWidget(parent):
+    from plugins.usermanagement.Usermanagement import Usermanagement
+    pluginWidget = Usermanagement(parent)
+    
+    return pluginWidget
+    
+###############################################################################
+
+def getPluginSettingsWidget(parent):
+    return None
+    
+###############################################################################
+
+def postprocess():
+    return
+    
+###############################################################################
 
 def addPreProcess(serverMeta, dn, objectValues, groupName):
     try:
